use crate::script::{Class, Locale};
use anyhow::Context;
use serde::Serialize;
use tera::Tera;

use crate::script::ScriptData;

pub fn render(sdata: ScriptData) -> anyhow::Result<Vec<u8>> {
    let days = sdata.days;
    let max_len = days.cols().iter().map(|c| c.len()).max().unwrap_or(0);

    let mut rows = vec![];
    for i in 0..max_len {
        rows.push(Row {
            idx: i + 1,
            time: sdata.times.get(i).cloned().unwrap_or_default(),

            mo: days.mo.get(i).cloned(),
            tu: days.tu.get(i).cloned(),
            we: days.we.get(i).cloned(),
            th: days.th.get(i).cloned(),
            fr: days.fr.get(i).cloned(),
        });
    }

    let rendered = Tera::one_off(
        include_str!("../assets/template.html.tera"),
        &tera::Context::from_serialize(RenderData {
            rows,
            times: !sdata.times.is_empty(),
            locale: sdata.locale,
            repeat: sdata.repeat_count,
        })
        .context("failed to serialize template data")?,
        false,
    )
    .context("failed to render template")?;

    Ok(rendered.into_bytes())
}

#[derive(Debug, Serialize)]
struct RenderData {
    rows: Vec<Row>,
    locale: Locale,
    repeat: usize,
    times: bool,
}

#[derive(Debug, Serialize)]
struct Row {
    idx: usize,
    time: String,
    mo: Option<Class>,
    tu: Option<Class>,
    we: Option<Class>,
    th: Option<Class>,
    fr: Option<Class>,
}
