use mlua::{DeserializeOptions, Lua, LuaSerdeExt, Table, Value};
use serde::{Deserialize, Serialize};
use std::{fs, path::PathBuf, sync::Arc};

use crate::renderer;

pub fn lua_render<'lua>(lua: &'lua Lua, args: Table<'lua>) -> Result<(), mlua::Error> {
    let args =
        lua.from_value_with::<ScriptData>(Value::Table(args), DeserializeOptions::default())?;
    let path = args.outfile.clone();

    let rendered =
        renderer::render(args).map_err(|e| mlua::Error::ExternalError(Arc::from(Box::from(e))))?;

    fs::write(path, rendered)?;

    Ok(())
}

#[derive(Debug, Deserialize)]
pub struct ScriptData {
    pub days: Days,
    pub outfile: PathBuf,
    #[serde(default)]
    pub locale: Locale,
    #[serde(default = "default_repeat_count")]
    pub repeat_count: usize,
    pub times: Vec<String>,
}

fn default_repeat_count() -> usize {
    1
}

#[derive(Debug, Deserialize)]
pub struct Days {
    pub mo: Vec<Class>,
    pub tu: Vec<Class>,
    pub we: Vec<Class>,
    pub th: Vec<Class>,
    pub fr: Vec<Class>,
}

impl Days {
    pub fn cols(&self) -> [&Vec<Class>; 5] {
        [&self.mo, &self.tu, &self.we, &self.th, &self.fr]
    }
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Class {
    pub room: Option<String>,
    pub name: String,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Locale {
    pub time: String,
    pub mo: String,
    pub tu: String,
    pub we: String,
    pub th: String,
    pub fr: String,
}

impl Default for Locale {
    fn default() -> Self {
        Self {
            time: "Time".into(),

            mo: "Monday".into(),
            tu: "Tuesday".into(),
            we: "Wednesday".into(),
            th: "Thursday".into(),
            fr: "Friday".into(),
        }
    }
}
